
# q-expansion of newform 1600.4.a.s, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1600
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [0], [6], [32], [-38], [-26], [100], [-78], [50], [108], [266], [22], [-442], [-514], [2], [500], [518], [-126], [-412], [878], [-600], [-282], [-150], [-386], [-702], [-598], [1194], [550], [-1562], [1846], [-2208], [2334], [-700], [-2050], [-1852], [-2494], [-2762], [3126], [-78], [-1300], [-1742], [-3772], [358], [-2214], [2600], [-1168], [-6478], [-646], [-3750], [-1482], [-1400], [3022], [-1248], [-2106], [-3638], [6550], [4388], [546], [-6858], [-9282], [4842], [2594], [-7332], [-1562], [1426], [-4008], [-8866], [1714], [-1150], [4398], [-1800], [-5874], [-2078], [7900], [-7518], [1950], [13786], [6402], [11150], [-13700], [5438], [-7692], [1118], [2600], [11958], [-17050], [9494], [11418], [7962], [-6526], [-17400], [1166], [7072], [100], [2602], [-11150], [-3638], [2078], [-5622], [-16486], [11706], [25038], [17550], [10712], [13654], [-14166], [-17842], [17600], [27302], [-3794], [-13238], [11574], [8300], [7508], [-27378], [-1842], [-10114], [10402], [7100], [7118], [31278], [-30054], [4518], [29272], [5798], [-8950], [-7800], [-8554], [2882], [18700], [12242], [31148], [-7694], [-4518], [-39550], [22122], [16634], [27586], [3850], [10032], [-20562], [10322], [-8846], [25350], [-46000], [-16998], [26494], [-21500], [25762], [30546], [32942], [27118], [-38634], [1794], [-41732], [-29200], [-48650], [11334], [31178], [-4686], [598], [41726], [24312], [-40946], [42282], [-1172], [-31614], [2450], [50362], [-12900], [-1462], [30108], [-8082], [-5200], [29850], [-46248], [24018], [-30238], [-35050], [27566], [-8528], [63442], [67214], [-40398], [-51350], [36426], [27278], [6350], [47548], [-2602], [-23162], [8712], [358], [-77766], [15158], [20302], [-25038], [-22626], [-56678], [66450], [-79692], [-75534], [-3950], [-56700], [40146], [-24800], [-43682], [-26550], [-42328], [6214], [-57202], [-49398], [-57206], [-30000], [6662], [-20954], [-13518], [-64874], [-13878], [45758], [-20000], [28050], [11722], [29354], [38350], [-17682], [-44800], [93886], [-77448], [82602], [-37100], [-68012], [71942], [34318], [-17034], [-89850], [-106958], [-58500], [-15732], [-84522], [36192], [47842], [46050], [51198], [-33800], [91526], [-57688], [114500], [-34918], [60586], [-25198], [-101394], [64350], [-42638], [-9900], [23138], [1954], [-58734], [-73706], [-82238], [55274], [74650], [19242], [-39786], [-5900], [-104750], [-28838], [34678], [58682], [102778], [-14686], [-102602], [78000], [139254], [-43918], [-68766], [-105150], [80502], [89232], [75922], [140108], [-38114], [97218], [-49926], [120588], [87878], [-24054], [120400], [40950], [-127402], [-113206], [151038], [30192], [59282], [-83850], [33148], [23322], [152700], [-89966], [-96642], [-66814], [96200], [88398], [149632], [24574], [-20046], [-67550], [-1800], [51402], [92562], [38250], [151842], [-59282], [-47834], [-74150], [-114300], [71868], [-70362], [9750], [-39208], [-90866], [-26522], [-36758], [-22002], [-98018], [156500], [116198], [-79794], [165362], [-85962], [-102334], [-57800], [161558], [101552], [-28726], [20950], [-53722], [-69658], [-62634], [-156758], [34214], [7950], [78468], [-67318], [-81900], [43678], [-41086], [-113652], [210106], [116112], [-142746], [125558], [53482], [177450], [35358], [-69400], [142632], [-28626], [102722], [-54734], [-59878], [-199314], [-103900], [42874], [181478], [49146], [107802], [-34138], [-80208], [128700], [-358], [-22050], [98148], [-187094], [-25900], [-75972], [-74642], [111750], [-18226], [-32662], [-81282], [-146114], [-13906], [161700], [-67438], [212788], [174746], [-234282], [-62034], [206550], [261842], [16100], [-79406], [136468], [162238], [92200], [-40950], [49792], [-227922], [-25450], [-160602], [62926], [249854], [189950], [-66172], [-171214], [-77198], [-201402], [-142500], [-212082], [113466], [-6242], [-32448], [-67106], [53818], [245600], [247366], [170814], [-278798], [-276250], [-53174], [174846], [-250300], [157798], [-214894], [191438], [750], [51512], [-260600]]
